@@ -1,5 +1,5 @@
 # -*- perl -*-
-# $Id: make-host-list.pl,v 1.3 1996/01/14 20:38:02 dm Exp $
+# $Id: make-host-list.pl,v 1.5 1996/01/31 07:36:23 dm Exp $
 
 # Much of this code is from make-ssh-known-hosts, by Tero Kivinen.
 # See make-ssh-known-hosts (which comes with ssh) for the copying
@@ -132,9 +132,8 @@ foreach $host (keys %host) {
     local ($namelist,$cnames, %ua);
     next unless $host =~ /.*$domain\.?$/;
     next if $host =~ /^localhost\./;
-    $addr = $addrlist = $host{$host};
-    $addr =~ s/,.*//;
-    $addrlist = join (',', grep (&inua ($host), split (/,/, $addrlist)));
+    $addrlist = $host{$host};
+    $addrlist = join (',', grep (&inua ($_), split (/,/, $addrlist)));
     $cnames = defined ($cname{$host}) ? "," . $cname{$host} : "";
     $namelist = "";
     unless ($opt_d) {
@@ -151,5 +150,5 @@ foreach $host (keys %host) {
 	    }
 	}
     }
-    print "$addr $host$cnames$namelist,$addrlist\n";
+    print "$addrlist $host$cnames$namelist,$addrlist\n";
 }
