@@ -1,4 +1,4 @@
-/* $Id: ssh-keyscan.c,v 1.4 1996/01/14 20:38:04 dm Exp $ */
+/* $Id: ssh-keyscan.c,v 1.6 1996/01/31 19:12:23 dm Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -11,15 +11,28 @@
 #include <sys/uio.h>
 #include <sys/resource.h>
 #include <netinet/in.h>
-#include <string.h>
 #include <errno.h>
 #include <ctype.h>
 #include <netdb.h>
+#include <string.h>
+
 #include "queue.h"
+
 #include <config.h>
 #include <gmp.h>
 #include <fdlim.h>
 
+#ifndef HAVE_STRCHR
+#define strchr index
+#define strrchr rindex
+#endif /* !HAVE_STRCHR */
+
+#ifdef HAVE_MEMCPY
+#define bzero(ptr, len) memset (ptr, 0, len)
+#define bcopy(src, dst, len) memcpy (dst, src, len)
+#define bcmp(ptr1, ptr2, len) memcmp (ptr1, ptr2, ptr3)
+#endif /* HAVE_MEMCPY */
+
 extern int errno;
 extern const char *const sys_errlist[];
 #define errmsg sys_errlist[errno]
@@ -58,7 +71,9 @@ typedef struct Connection {
   int c_plen;                   /* Packet length field for ssh packet */
   int c_len;                    /* Total bytes which must be read. */
   int c_off;                    /* Length of data read so far. */
+  char *c_namebase;             /* Address to free for c_name and c_namelist */
   char *c_name;                 /* Hostname of connection for errors */
+  char *c_namelist;             /* Pointer to other possible addresses */
   char *c_output_name;          /* Hostname of connection for output */
   char *c_data;                 /* Data read from this fd */
   struct timeval c_tv;          /* Time at which connection gets aborted */
@@ -114,6 +129,42 @@ xstrdup (char *s)
   return (d);
 }
 
+/* This is an strsep function that returns a null field for adjacent
+ * separators.  This is the same as the 4.4BSD strsep, but different
+ * from the one in the GNU libc. */
+inline char *
+xstrsep(char **str, const char *delim)
+{
+  char *s, *e;
+
+  if (!**str)
+    return (NULL);
+
+  s = *str;
+  e = s + strcspn (s, delim);
+
+  if (*e != '\0')
+    *e++ = '\0';
+  *str = e;
+
+  return (s);
+}
+
+/* Get the next non-null token (like GNU strsep).  Strsep() will
+ * return a null token for two adjacent separators, so we may have to
+ * loop. */
+char *
+strnnsep (char **stringp, char *delim)
+{
+  char *tok;
+
+  do {
+    tok = xstrsep (stringp, delim);
+  } while (tok && *tok == '\0');
+  return (tok);
+}
+
+
 unsigned char
 getbyte (char **cpp, char *eom)
 {
@@ -260,18 +311,21 @@ tcpconnect (char *hname)
 }
 
 int
-conalloc (char *hname)
+conalloc (char *iname, char *oname)
 {
   int s;
-  char *p;
+  char *namebase, *name, *namelist;
 
-  p = strtok (hname, " \t\n");
-  if (p == NULL)
-    return (-1);
+  namebase = namelist = xstrdup (iname);
+
+  do {
+    name = xstrsep (&namelist, ",");
+    if (!name) {
+      free (namebase);
+      return (-1);
+    }
+  } while ((s = tcpconnect (name)) < 0);
 
-  s = tcpconnect (p);
-  if (s < 0)
-    return (-1);
   if (s >= maxfd)
     fatal ("conalloc: fdno %d too high\n", s);
   if (fdcon[s].c_status)
@@ -279,9 +333,10 @@ conalloc (char *hname)
 
   fdcon[s].c_fd = s;
   fdcon[s].c_status = CS_CON;
-  fdcon[s].c_name = xstrdup (p);
-  p = strtok (NULL, " \t\n");
-  fdcon[s].c_output_name = p ? xstrdup (p) : NULL;
+  fdcon[s].c_namebase = namebase;
+  fdcon[s].c_name = name;
+  fdcon[s].c_namelist = namelist;
+  fdcon[s].c_output_name = xstrdup (oname);
   fdcon[s].c_data = (char *) &fdcon[s].c_plen;
   fdcon[s].c_len = 4;
   fdcon[s].c_off = 0;
@@ -299,7 +354,8 @@ confree (int s)
   close (s);
   if (s >= maxfd || fdcon[s].c_status == CS_UNUSED)
     fatal ("confree: attempt to free bad fdno %d\n", s);
-  free (fdcon[s].c_name);
+  free (fdcon[s].c_namebase);
+  free (fdcon[s].c_output_name);
   if (fdcon[s].c_status == CS_KEYS)
     free (fdcon[s].c_data);
   fdcon[s].c_status = CS_UNUSED;
@@ -317,6 +373,22 @@ contouch (int s)
   TAILQ_INSERT_TAIL (&tq, &fdcon[s], c_link);
 }
 
+int
+conrecycle (int s)
+{
+  int ret;
+  con *c = &fdcon[s];
+  char *iname, *oname;
+
+  iname = xstrdup (c->c_namelist);
+  oname = c->c_output_name;
+  c->c_output_name = NULL;  /* prevent it from being freed */
+  confree (s);
+  ret = conalloc (iname, oname);
+  free (iname);
+  return (ret);
+}
+
 void
 congreet (int s)
 {
@@ -328,7 +400,7 @@ congreet (int s)
   if (n < 0) {
     if (errno != ECONNREFUSED)
       warn ("read (%s): %s\n", c->c_name, errmsg);
-    confree (s);
+    conrecycle (s);
     return;
   }
   if (buf[n - 1] != '\n') {
@@ -427,7 +499,7 @@ conloop (void)
 		   && c->c_tv.tv_usec < now.tv_usec))) {
     int s = c->c_fd;
     c = c->c_link.tqe_next;
-    confree (s);
+    conrecycle (s);
   }
 }
 
@@ -525,10 +597,13 @@ main (int argc, char **argv)
 
   do {
     while (ncon < maxcon) {
+      char *name;
+
       host = nexthost (argc, argv);
       if (host == NULL)
 	break;
-      conalloc (host);
+      name = strnnsep (&host, " \t\n");
+      conalloc (name, *host ? host : name);
     }
     conloop ();
   } while (host);
